@@ -14,6 +14,8 @@ from django.db.models import Q
 from django.forms import ValidationError
 from django.template.loader import render_to_string
 from django.template import RequestContext
+from django.core import mail
+from django.conf import settings as main_settings
 
 
 # Create your views here.
@@ -132,15 +134,24 @@ def reminders(request):
     today, now = get_today_now()
     tomorrow = today + datetime.timedelta(days=1)
 
-    user = request.user
-    queryset = (                            # customers see their own entries
-        Entry.objects.filter(
+    # get the customer corresponding to the user unless user is staff
+    customer = (
+        Customer.objects.get(
+            username=request.user.username
+        ) if not request.user.is_staff
+        else
+            request.user
+    )
+
+    queryset = (
+        Entry.objects.filter(          # admin/staff users see everything
             Q(date=today, time__gte=now)|Q(date=tomorrow),
-            customer=user,
+        ) if request.user.is_staff
+        else
+        Entry.objects.filter(          # customers only see their own entries
+            Q(date=today, time__gte=now)|Q(date=tomorrow),
+            customer=customer,
             cancelled=False,
-        ) if isinstance(user, Customer)
-        else Entry.objects.filter(          # admin/staff users see everything
-            Q(date=today, time__gte=now)|Q(date=tomorrow),
         )
     )
     return queryset.order_by('date', 'time')
@@ -369,13 +380,6 @@ def multi_day(request, slug=None, change=None):
                     date=day,
                     time__gte=startTime,
                     time__lt=endTime,
-                ) if request.user.is_staff
-                else Entry.objects.filter(
-                    date=day,
-                    time__gte=startTime,
-                    time__lt=endTime,
-                    customer=request.user,
-                    cancelled=False,
                 )
             ).order_by('time')
 
@@ -433,13 +437,6 @@ def day(request, slug=None, change=None):
                 date=date,
                 time__gte=startTime,
                 time__lt=endTime,
-            ) if request.user.is_staff
-            else Entry.objects.filter(
-                date=date,
-                time__gte=startTime,
-                time__lt=endTime,
-                customer=request.user,
-                cancelled=False,
             )
         ).order_by('time')
 
@@ -505,6 +502,38 @@ def get_redirect_url(request, default):
     return '/'.join(x for x in next_url_bits[:date_index]) + '/' + slug + '/'
 
 
+def email_notify_entry_change(entry, header_text, body_text_filename, context):
+    """
+    Notify admins and clients of changes in the diary entries by email.
+    """
+    # abort if not required
+    if not entry.customer or (      # don't bother if no customer is set
+        not main_settings.ADMINS and not ( # don't bother if noone wants email
+            entry.customer.email and not entry.customer.opt_out_entry_change_email
+            )
+        ):
+        return
+
+    # construct the list of recipients
+    recipients = []
+    for name, email in main_settings.ADMINS: # the admins
+        recipients.append(email)
+
+    if entry.customer.email and not entry.customer.opt_out_entry_change_email:
+        recipients.append(entry.customer.email)
+
+    mail.send_mail(
+        settings.DIARY_SITE_NAME+header_text.format(entry.customer),
+        render_to_string(
+            body_text_filename,
+            context=context,
+        ),
+        main_settings.DEFAULT_FROM_EMAIL,
+        recipients,
+            )
+
+
+
 @login_required
 def entry(request, pk=None, slug=None, customer_pk=None):
     """
@@ -552,6 +581,15 @@ def entry(request, pk=None, slug=None, customer_pk=None):
     if form.is_valid():
         entry = form.save(commit=False)
         entry.save()
+
+        # notify changes by email
+        email_notify_entry_change(
+            entry,
+            ': New Diary Entry for {}',
+            'diary/email_notify_new_entry.txt',
+            {'entry': entry,},
+        )
+
         return redirect(next_url)
 
     # have to set up the customer widget after form creation
@@ -605,6 +643,8 @@ def entry_update(request):
     date, time = getDatetimeFromSlug(datetime_slug)
 
     # try updating the entry
+    old_entry_date = entry.date
+    old_entry_time = entry.time
     entry.date = date
     entry.time = time
 
@@ -617,7 +657,7 @@ def entry_update(request):
     except ValidationError as ve:
         # attempt to fit the entry in later in the time slot
         endTime = (
-            datetime.datetime.combine(date, time) + TIME_INC
+            datetime.datetime.combine(date, time) + DIARY_TIME_INC
             ).time()
         otherEntry = Entry.objects\
             .filter(
@@ -632,6 +672,18 @@ def entry_update(request):
         entry.time = otherEntry.time_end()
         entry.save()
 
+    # notify changes by email
+    email_notify_entry_change(
+        entry,
+        ': Diary Entry Change for {}',
+        'diary/email_notify_entry_change.txt',
+        {
+            'entry': entry,
+            'old_entry_date': old_entry_date,
+            'old_entry_time': old_entry_time,
+        },
+    )
+
     message = 'Date / time changed to {0}, {1}'.format(entry.date, entry.time)
     data = {'message': message}
     #print('Sending entry_update data {0}'.format(data))
@@ -723,6 +775,17 @@ def entry_admin(request, pk, action):
         entry.editor = request.user
         entry.save()
 
+    # notify entry status change by email
+    email_notify_entry_change(
+        entry,
+        ': Diary Entry Status Change for {}',
+        'diary/email_notify_entry_status_change.txt',
+        {
+            'entry': entry,
+            'status': action,
+        },
+    )
+
     return redirect(
         redirect_url,
         slug=date.strftime(DATE_SLUG_FORMAT),
@@ -760,9 +823,9 @@ def customer_add(request, entry_pk=None, entry_slug=None):
                     }
                 )
             )
-        elif not request.user.is_authenticated():
+        elif not request.user.is_authenticated:
             return HttpResponseRedirect(
-                reverse('django.contrib.auth.views.login')
+                reverse('login')
             )
         else:
             return HttpResponseRedirect(
@@ -794,7 +857,7 @@ def get_customer_and_redirect(request, pk):
 
     customer = (
         Customer.objects.get(pk=pk) if (pk and request.user.is_staff)
-        else Customer.objects.get(request.user.pk)
+        else Customer.objects.get(pk=request.user.pk)
     )
 
     redirect_url = (
@@ -805,11 +868,13 @@ def get_customer_and_redirect(request, pk):
     return customer, redirect_url
 
 
-
 @login_required
-def customer_change(request, pk):
+def customer_change(request, pk=None):
     """
     Change the specified customer's personal details.
+
+    The default pk points to nothing and forces use of the logged
+    user's pk.
     """
 
     customer, redirect_url = get_customer_and_redirect(request, pk)
